/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
import { ScopeType, type IUser } from "@fluidframework/protocol-definitions";
import { IGitManager } from "@fluidframework/server-services-client";
import * as core from "@fluidframework/server-services-core";
import { RawAxiosRequestHeaders } from "axios";
export declare function getRefreshTokenIfNeededCallback(tenantManager: core.ITenantManager, documentId: string, tenantId: string, scopes: ScopeType[], serviceName: string): (authorizationHeader: RawAxiosRequestHeaders) => Promise<RawAxiosRequestHeaders | undefined>;
/**
 * @internal
 */
export declare class Tenant implements core.ITenant {
    private readonly config;
    private readonly manager;
    get id(): string;
    get gitManager(): IGitManager;
    get storage(): core.ITenantStorage;
    get orderer(): core.ITenantOrderer;
    constructor(config: core.ITenantConfig, manager: IGitManager);
}
/**
 * Manages a collection of tenants
 * @internal
 */
export declare class TenantManager implements core.ITenantManager, core.ITenantConfigManager {
    private readonly endpoint;
    private readonly internalHistorianUrl;
    private readonly invalidTokenCache?;
    constructor(endpoint: string, internalHistorianUrl: string, invalidTokenCache?: core.ICache | undefined);
    createTenant(tenantId?: string): Promise<core.ITenantConfig & {
        key: string;
    }>;
    getTenant(tenantId: string, documentId: string, includeDisabledTenant?: boolean): Promise<core.ITenant>;
    getTenantGitManager(tenantId: string, documentId: string, storageName?: string, includeDisabledTenant?: boolean, isEphemeralContainer?: boolean): Promise<IGitManager>;
    private throwInvalidTokenErrorAsNetworkError;
    verifyToken(tenantId: string, token: string): Promise<void>;
    getKey(tenantId: string, includeDisabledTenant?: boolean): Promise<string>;
    signToken(tenantId: string, documentId: string, scopes: ScopeType[], user?: IUser, lifetime?: number, ver?: string, jti?: string, includeDisabledTenant?: boolean): Promise<string>;
    getTenantStorageName(tenantId: string, includeDisabledTenant?: boolean): Promise<string>;
    private getTenantConfig;
}
//# sourceMappingURL=tenant.d.ts.map