"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TenantManager = exports.Tenant = exports.getRefreshTokenIfNeededCallback = void 0;
const protocol_definitions_1 = require("@fluidframework/protocol-definitions");
const server_services_client_1 = require("@fluidframework/server-services-client");
const core = __importStar(require("@fluidframework/server-services-core"));
const common_utils_1 = require("@fluidframework/common-utils");
const server_services_utils_1 = require("@fluidframework/server-services-utils");
const server_services_telemetry_1 = require("@fluidframework/server-services-telemetry");
const _1 = require(".");
function getRefreshTokenIfNeededCallback(tenantManager, documentId, tenantId, scopes, serviceName) {
    const refreshTokenIfNeeded = async (authorizationHeader) => {
        if (authorizationHeader.Authorization &&
            typeof authorizationHeader.Authorization === "string") {
            const currentAccessToken = (0, server_services_utils_1.extractTokenFromHeader)(authorizationHeader.Authorization);
            const props = {
                ...(0, server_services_telemetry_1.getLumberBaseProperties)(documentId, tenantId),
                serviceName,
                scopes,
            };
            const newAccessToken = await (0, server_services_utils_1.getValidAccessToken)(currentAccessToken, tenantManager, tenantId, documentId, scopes, props).catch((error) => {
                server_services_telemetry_1.Lumberjack.error("Failed to refresh access token", props, error);
                throw error;
            });
            if (newAccessToken) {
                return {
                    Authorization: `Basic ${newAccessToken}`,
                };
            }
            return undefined;
        }
    };
    return refreshTokenIfNeeded;
}
exports.getRefreshTokenIfNeededCallback = getRefreshTokenIfNeededCallback;
/**
 * @internal
 */
class Tenant {
    get id() {
        return this.config.id;
    }
    get gitManager() {
        return this.manager;
    }
    get storage() {
        return this.config.storage;
    }
    get orderer() {
        return this.config.orderer;
    }
    constructor(config, manager) {
        this.config = config;
        this.manager = manager;
    }
}
exports.Tenant = Tenant;
/**
 * Manages a collection of tenants
 * @internal
 */
class TenantManager {
    constructor(endpoint, internalHistorianUrl, invalidTokenCache) {
        this.endpoint = endpoint;
        this.internalHistorianUrl = internalHistorianUrl;
        this.invalidTokenCache = invalidTokenCache;
    }
    async createTenant(tenantId) {
        const restWrapper = new server_services_client_1.BasicRestWrapper(this.endpoint /* baseUrl */, undefined /* defaultQueryString */, undefined /* maxBodyLength */, undefined /* maxContentLength */, undefined /* defaultHeaders */, undefined /* axios */, undefined /* refreshDefaultQureyString */, undefined /* refreshDefaultHeaders */, () => (0, server_services_telemetry_1.getGlobalTelemetryContext)().getProperties().correlationId, () => (0, server_services_telemetry_1.getGlobalTelemetryContext)().getProperties(), undefined /* refreshTokenIfNeeded */, server_services_utils_1.logHttpMetrics);
        const result = await restWrapper.post(`/api/tenants/${encodeURIComponent(tenantId || "")}`, undefined);
        return result;
    }
    async getTenant(tenantId, documentId, includeDisabledTenant = false) {
        const [details, gitManager] = await Promise.all([
            this.getTenantConfig(tenantId, includeDisabledTenant),
            this.getTenantGitManager(tenantId, documentId, undefined, includeDisabledTenant),
        ]);
        const tenant = new Tenant(details, gitManager);
        return tenant;
    }
    async getTenantGitManager(tenantId, documentId, storageName, includeDisabledTenant = false, isEphemeralContainer = false) {
        const lumberProperties = {
            ...(0, server_services_telemetry_1.getLumberBaseProperties)(documentId, tenantId),
            [server_services_telemetry_1.CommonProperties.isEphemeralContainer]: isEphemeralContainer,
        };
        const scopes = [protocol_definitions_1.ScopeType.DocWrite, protocol_definitions_1.ScopeType.DocRead, protocol_definitions_1.ScopeType.SummaryWrite];
        const accessToken = await core.requestWithRetry(async () => this.signToken(tenantId, documentId, scopes), "getTenantGitManager_signToken" /* callName */, lumberProperties /* telemetryProperties */);
        const defaultQueryString = {
            token: (0, common_utils_1.fromUtf8ToBase64)(`${tenantId}`),
        };
        const getDefaultHeaders = () => {
            const credentials = {
                password: accessToken,
                user: tenantId,
            };
            const headers = {
                Authorization: (0, server_services_client_1.getAuthorizationTokenFromCredentials)(credentials),
            };
            if (storageName) {
                headers.StorageName = storageName;
            }
            // IsEphemeralContainer header is set only for ephemeral containers
            // It is not set if it is not ephemeral and when the driver did not send any info
            if (isEphemeralContainer) {
                headers[_1.IsEphemeralContainer] = isEphemeralContainer;
            }
            return headers;
        };
        const refreshTokenIfNeeded = async (authorizationHeader) => {
            if (authorizationHeader.Authorization &&
                typeof authorizationHeader.Authorization === "string") {
                const currentAccessToken = (0, server_services_client_1.parseToken)(tenantId, authorizationHeader.Authorization);
                if (currentAccessToken) {
                    const props = {
                        ...lumberProperties,
                        serviceName: "historian",
                        scopes,
                    };
                    const tenantManager = new TenantManager(this.endpoint, this.internalHistorianUrl);
                    const newAccessToken = await (0, server_services_utils_1.getValidAccessToken)(currentAccessToken, tenantManager, tenantId, documentId, scopes, props).catch((error) => {
                        server_services_telemetry_1.Lumberjack.error("Failed to refresh access token", props, error);
                        throw error;
                    });
                    if (newAccessToken) {
                        const newCredentials = {
                            password: newAccessToken,
                            user: tenantId,
                        };
                        return {
                            Authorization: (0, server_services_client_1.getAuthorizationTokenFromCredentials)(newCredentials),
                        };
                    }
                    return undefined;
                }
            }
        };
        const defaultHeaders = getDefaultHeaders();
        const baseUrl = `${this.internalHistorianUrl}/repos/${encodeURIComponent(tenantId)}`;
        const tenantRestWrapper = new server_services_client_1.BasicRestWrapper(baseUrl, defaultQueryString, undefined, undefined, defaultHeaders, undefined, undefined, getDefaultHeaders, () => (0, server_services_telemetry_1.getGlobalTelemetryContext)().getProperties().correlationId, () => (0, server_services_telemetry_1.getGlobalTelemetryContext)().getProperties(), refreshTokenIfNeeded, server_services_utils_1.logHttpMetrics);
        const historian = new server_services_client_1.Historian(baseUrl, true, false, tenantRestWrapper);
        const gitManager = new server_services_client_1.GitManager(historian);
        return gitManager;
    }
    throwInvalidTokenErrorAsNetworkError(cachedInvalidTokenError) {
        try {
            const errorObject = JSON.parse(cachedInvalidTokenError);
            const networkError = new server_services_client_1.NetworkError(errorObject.code, errorObject.message, false, false, undefined, "InvalidTokenCache");
            throw networkError;
        }
        catch (error) {
            if ((0, server_services_client_1.isNetworkError)(error)) {
                throw error;
            }
            // Do not throw an error if the cached invalid token error is not a valid JSON
            server_services_telemetry_1.Lumberjack.error("Failed to parse cached invalid token error", {}, error);
        }
    }
    async verifyToken(tenantId, token) {
        if (this.invalidTokenCache) {
            const cachedInvalidTokenError = await this.invalidTokenCache.get(token);
            if (cachedInvalidTokenError) {
                this.throwInvalidTokenErrorAsNetworkError(cachedInvalidTokenError);
            }
        }
        const restWrapper = new server_services_client_1.BasicRestWrapper(this.endpoint /* baseUrl */, undefined /* defaultQueryString */, undefined /* maxBodyLength */, undefined /* maxContentLength */, undefined /* defaultHeaders */, undefined /* axios */, undefined /* refreshDefaultQureyString */, undefined /* refreshDefaultHeaders */, () => (0, server_services_telemetry_1.getGlobalTelemetryContext)().getProperties().correlationId, () => (0, server_services_telemetry_1.getGlobalTelemetryContext)().getProperties(), undefined /* refreshTokenIfNeeded */, server_services_utils_1.logHttpMetrics);
        try {
            await restWrapper.post(`/api/tenants/${encodeURIComponent(tenantId)}/validate`, {
                token,
            });
        }
        catch (error) {
            if ((0, server_services_client_1.isNetworkError)(error)) {
                // In case of a 401 or 403 error, we cache the token in the invalid token cache
                // to avoid hitting the endpoint again with the same token.
                if (error.code === 401 || error.code === 403) {
                    const errorToCache = {
                        code: error.code,
                        message: error.message,
                    };
                    // Cache the token in the invalid token cache
                    // to avoid hitting the endpoint again with the same token.
                    this.invalidTokenCache
                        ?.set(token, JSON.stringify(errorToCache))
                        .catch((err) => {
                        server_services_telemetry_1.Lumberjack.error("Failed to set token in invalid token cache", { tenantId }, err);
                    });
                }
            }
            throw error;
        }
    }
    async getKey(tenantId, includeDisabledTenant = false) {
        const restWrapper = new server_services_client_1.BasicRestWrapper(this.endpoint /* baseUrl */, undefined /* defaultQueryString */, undefined /* maxBodyLength */, undefined /* maxContentLength */, undefined /* defaultHeaders */, undefined /* axios */, undefined /* refreshDefaultQureyString */, undefined /* refreshDefaultHeaders */, () => (0, server_services_telemetry_1.getGlobalTelemetryContext)().getProperties().correlationId, () => (0, server_services_telemetry_1.getGlobalTelemetryContext)().getProperties(), undefined /* refreshTokenIfNeeded */, server_services_utils_1.logHttpMetrics);
        const result = await restWrapper.get(`/api/tenants/${encodeURIComponent(tenantId)}/keys`, { includeDisabledTenant });
        return result.key1;
    }
    async signToken(tenantId, documentId, scopes, user, lifetime, ver, jti, includeDisabledTenant) {
        const restWrapper = new server_services_client_1.BasicRestWrapper(this.endpoint /* baseUrl */, undefined /* defaultQueryString */, undefined /* maxBodyLength */, undefined /* maxContentLength */, undefined /* defaultHeaders */, undefined /* axios */, undefined /* refreshDefaultQueryString */, undefined /* refreshDefaultHeaders */, () => (0, server_services_telemetry_1.getGlobalTelemetryContext)().getProperties().correlationId, () => (0, server_services_telemetry_1.getGlobalTelemetryContext)().getProperties(), undefined /* refreshTokenIfNeeded */, server_services_utils_1.logHttpMetrics);
        const result = await restWrapper.post(`/api/tenants/${encodeURIComponent(tenantId)}/accesstoken`, {
            documentId,
            scopes,
            user,
            lifetime,
            ver,
            jti,
        }, { includeDisabledTenant: includeDisabledTenant ?? false });
        return result.fluidAccessToken;
    }
    async getTenantStorageName(tenantId, includeDisabledTenant = false) {
        const tenantConfig = await this.getTenantConfig(tenantId, includeDisabledTenant);
        return tenantConfig?.customData?.storageName;
    }
    async getTenantConfig(tenantId, includeDisabledTenant = false) {
        const restWrapper = new server_services_client_1.BasicRestWrapper(this.endpoint /* baseUrl */, undefined /* defaultQueryString */, undefined /* maxBodyLength */, undefined /* maxContentLength */, undefined /* defaultHeaders */, undefined /* axios */, undefined /* refreshDefaultQureyString */, undefined /* refreshDefaultHeaders */, () => (0, server_services_telemetry_1.getGlobalTelemetryContext)().getProperties().correlationId, () => (0, server_services_telemetry_1.getGlobalTelemetryContext)().getProperties(), undefined /* refreshTokenIfNeeded */, server_services_utils_1.logHttpMetrics);
        return restWrapper.get(`/api/tenants/${tenantId}`, {
            includeDisabledTenant,
        });
    }
}
exports.TenantManager = TenantManager;
//# sourceMappingURL=tenant.js.map